/***
 * FractalGUI: a graphical tool to edit Fractal component configurations.
 * Copyright (C) 2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Authors: Eric Bruneton, Patrice Fauvel
 */

package org.objectweb.fractal.gui.repository.lib;

import org.objectweb.fractal.api.control.BindingController;

import org.objectweb.fractal.adl.ResolvingLoader;

import org.objectweb.fractal.gui.model.Component;
import org.objectweb.fractal.gui.model.Factory;
import org.objectweb.fractal.gui.repository.api.Repository;
import org.objectweb.fractal.gui.repository.api.Storage;

/**
 * Basic implementation of the {@link Repository} interface. This implementation
 * supposes that the component definitions are stored in the repository as
 * Fractal ADL definitions, in XML form.
 */

public class BasicRepository implements Repository, BindingController {

  /**
   * A mandatory client interface bound to a {@link Storage storage}. This is
   * the storage from which this repository loads the component definitions.
   */

  public final static String STORAGE_BINDING = "storage";

  /**
   * A mandatory client interface bound to a {@link Factory factory}. This
   * factory is used to create components from the component definitions stored
   * in the storage.
   */

  public final static String FACTORY_BINDING = "configuration-factory";

  /**
   * The storage client interface.
   */

  private Storage storage;

  /**
   * The factory client interface.
   */

  private Factory factory;

  // -------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // -------------------------------------------------------------------------

  public String[] listFc () {
    return new String[] { STORAGE_BINDING, FACTORY_BINDING };
  }

  public Object lookupFc (final String clientItfName) {
    if (STORAGE_BINDING.equals(clientItfName)) {
      return storage;
    } else if (FACTORY_BINDING.equals(clientItfName)) {
      return factory;
    }
    return null;
  }

  public void bindFc (
    final String clientItfName,
    final Object serverItf)
  {
    if (STORAGE_BINDING.equals(clientItfName)) {
      storage = (Storage)serverItf;
    } else if (FACTORY_BINDING.equals(clientItfName)) {
      factory = (Factory)serverItf;
    }
  }

  public void unbindFc (final String clientItfName) {
    if (STORAGE_BINDING.equals(clientItfName)) {
      storage = null;
    } else if (FACTORY_BINDING.equals(clientItfName)) {
      factory = null;
    }
  }

  // -------------------------------------------------------------------------
  // Implementation of the Repository interface
  // -------------------------------------------------------------------------

  public Component loadComponent (final String name) throws Exception {
    StorageLoader storageLoader = new StorageLoader();
    storageLoader.storage = storage;
    ResolvingLoader resolvingLoader = new  ResolvingLoader();
    resolvingLoader.bindFc("delegate", storageLoader);
    FractalAdlReader reader = new FractalAdlReader();
    reader.loader = resolvingLoader;
    reader.factory = factory;
    return reader.loadTemplate(name);
  }

  public String storeComponent (final Component component) throws Exception {
    FractalAdlWriter writer = new FractalAdlWriter();
    writer.storage = storage;
    return writer.saveTemplate(component);
  }
}
