/***
 * FractalGUI: a graphical tool to edit Fractal component configurations.
 * Copyright (C) 2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Authors: Eric Bruneton, Patrice Fauvel
 */

package org.objectweb.fractal.gui.repository.lib;

import org.objectweb.fractal.adl.XMLElement;

import org.objectweb.fractal.gui.model.Component;
import org.objectweb.fractal.gui.model.Interface;
import org.objectweb.fractal.gui.model.ClientInterface;
import org.objectweb.fractal.gui.model.ServerInterface;
import org.objectweb.fractal.gui.repository.api.Storage;

import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;

/**
 * A converter from {@link Component}s to Fractal ADL definitions. This
 * converter assigns generated names to components and interfaces whose name is
 * missing, in order to get valid Fractal ADL definitions.
 */

class FractalAdlWriter {

  /**
   * The storage used by this converter to store the converted definitions.
   */

  Storage storage;

  /**
   * Stores the definitions converted from the given component in the storage.
   *
   * @param c a component.
   * @return the name of the definition converted from c.
   * @throws Exception if something goes wrong.
   */

  public String saveTemplate (final Component c) throws Exception {
    Map types = new HashMap();
    Set shared = new HashSet();
    return saveTemplate(c, types, shared);
  }

  /**
   * Stores the definitions converted from the given component in the storage.
   *
   * @param c a component.
   * @param types a map to collect the {@link ComponentType}s of components.
   * @param shared a set to collect the slave components found in c.
   * @return the name of the definition converted from c.
   * @throws Exception if something goes wrong.
   */

  public String saveTemplate (
    final Component c,
    final Map types,
    final Set shared) throws Exception
  {
    if (c.isComposite()) {
      return saveCompositeTemplate(c, types, shared);
    } else {
      return savePrimitiveTemplate(c, types, shared);
    }
  }

  /**
   * Stores the definition converted from the given primitive component in the
   * storage.
   *
   * @param c a primitive component.
   * @param types a map to collect the {@link ComponentType}s of components.
   * @param shared a set to collect the slave components found in c.
   * @return the name of the definition converted from c.
   * @throws Exception if something goes wrong.
   */

  public String savePrimitiveTemplate (
    final Component c,
    final Map types,
    final Set shared) throws Exception
  {
    XMLElement tmpl = new XMLElement("primitive-template");
    tmpl.setAttribute("name", getComponentType(c));
    tmpl.setAttribute("implements", saveType(c, types));
    if (c.getImplementation().length() > 0) {
      XMLElement impl = new XMLElement("primitive-content");
      impl.setAttribute("class", c.getImplementation());
      tmpl.addSubElement(impl);
    }
    saveController(c, tmpl);
    storage.store(getComponentType(c), tmpl);
    return getComponentType(c);
  }

  /**
   * Stores the definitions converted from the given composite component in the
   * storage.
   *
   * @param c a composite component.
   * @param types a map to collect the {@link ComponentType}s of components.
   * @param shared a set to collect the slave components found in c.
   * @return the name of the definition converted from c.
   * @throws Exception if something goes wrong.
   */

  public String saveCompositeTemplate (
    final Component c,
    final Map types,
    final Set shared) throws Exception
  {
    XMLElement tmpl = new XMLElement("composite-template");
    tmpl.setAttribute("name", getComponentType(c));
    tmpl.setAttribute("implements", saveType(c, types));
    XMLElement content = new XMLElement("composite-content");

    // sub components
    XMLElement comps = new XMLElement("components");
    List subComps = c.getSubComponents();
    for (int i = 0; i < subComps.size(); ++i) {
      Component sc = (Component)subComps.get(i);
      XMLElement comp = new XMLElement("component");
      comp.setAttribute("name", getComponentName(sc));
      if (sc.getMasterComponent() == null) {
        comp.setAttribute("type", saveType(sc, types));
        comp.setAttribute("implementation", getComponentType(sc));
        saveTemplate(sc, types, shared);
      } else {
        comp.setAttribute("type", saveType(sc.getMasterComponent(), types));
        shared.add(sc);
      }
      comps.addSubElement(comp);
    }
    content.addSubElement(comps);

    // bindings
    XMLElement bindings = new XMLElement("bindings");
    List itfs = c.getServerInterfaces();
    for (int i = 0; i < itfs.size(); ++i) {
      ServerInterface sitf = (ServerInterface)itfs.get(i);
      ClientInterface citf = (ClientInterface)sitf.getComplementaryInterface();
      if (citf.getBinding() != null) {
        XMLElement binding = new XMLElement("binding");
        binding.setAttribute("client", "this." + getInterfaceName(citf));
        sitf = citf.getBinding().getServerInterface();
        if (sitf.getOwner() == c) {
          binding.setAttribute("server", "this." + getInterfaceName(sitf));
        } else {
          binding.setAttribute(
            "server",
            getComponentName(sitf.getOwner()) + "." + getInterfaceName(sitf));
        }
        bindings.addSubElement(binding);
      }
    }
    for (int i = 0; i < subComps.size(); ++i) {
      Component sc = (Component)subComps.get(i);
      if (sc.getMasterComponent() != null) {
        itfs = sc.getMasterComponent().getClientInterfaces();
      } else {
        itfs = sc.getClientInterfaces();
      }
      for (int j = 0; j < itfs.size(); ++j) {
        ClientInterface citf = (ClientInterface)itfs.get(j);
        if (citf.getBinding() != null) {
          XMLElement binding = new XMLElement("binding");
          binding.setAttribute(
            "client",
            getComponentName(sc) + "." + getInterfaceName(citf));
          Interface sitf = citf.getBinding().getServerInterface();
          if (sitf.getOwner() == c) {
            binding.setAttribute("server", "this." + getInterfaceName(sitf));
          } else if (sitf.getOwner().getParent() != c) {
            continue;
          } else {
            binding.setAttribute(
              "server",
              getComponentName(sitf.getOwner()) + "." + getInterfaceName(sitf));
          }
          bindings.addSubElement(binding);
        }
      }
    }
    if (bindings.getSubElementCount() > 0) {
      content.addSubElement(bindings);
    }

    // shared components
    XMLElement sharing = new XMLElement("sharing");
    Iterator i = shared.iterator();
    while (i.hasNext()) {
      Component slave = (Component)i.next();
      Component master = slave.getMasterComponent();
      if (c.contains(master) && c.contains(slave)) {
        String slavePath = getComponentName(slave);
        while (slave.getParent() != c) {
          slave = slave.getParent();
          slavePath = getComponentName(slave) + "/" + slavePath;
        }
        String masterPath = getComponentName(master);
        while (master.getParent() != c) {
          master = master.getParent();
          masterPath = getComponentName(master) + "/" + masterPath;
        }
        XMLElement share = new XMLElement("shared-component");
        share.setAttribute("path", slavePath);
        share.setAttribute("ref", masterPath);
        sharing.addSubElement(share);
        i.remove();
      }
    }
    if (sharing.getSubElementCount() > 0) {
      content.addSubElement(sharing);
    }

    tmpl.addSubElement(content);
    saveController(c, tmpl);
    storage.store(getComponentType(c), tmpl);
    return getComponentType(c);
  }

  // -------------------------------------------------------------------------
  // Controller part
  // -------------------------------------------------------------------------

  /**
   * Converts the controller descriptors of the given component.
   *
   * @param c a component
   * @param template the XML element to which the converted descriptors must be
   *      added.
   */

  public void saveController (final Component c, final XMLElement template) {
    XMLElement controller = new XMLElement("controller");
    saveAttributes(c, controller);
    if (c.getTemplateControllerDescriptor().length() > 0) {
      XMLElement tmplDesc = new XMLElement("template-controller");
      tmplDesc.setAttribute("desc", c.getTemplateControllerDescriptor());
      controller.addSubElement(tmplDesc);
    }
    if (c.getComponentControllerDescriptor().length() > 0) {
      XMLElement compDesc = new XMLElement("component-controller");
      compDesc.setAttribute("desc", c.getComponentControllerDescriptor());
      controller.addSubElement(compDesc);
    }
    if (controller.getSubElementCount() > 0) {
      template.addSubElement(controller);
    }
  }

  /**
   * Converts the attributes of the given component.
   *
   * @param c a component
   * @param controller the XML element to which the converted attributes must be
   *      added.
   */

  public void saveAttributes (final Component c, final XMLElement controller) {
    String attrController = c.getAttributeController();
    List attrNames = c.getAttributeNames();
    if (attrController.length() > 0 || attrNames.size() > 0) {
      XMLElement attrs = new XMLElement("attributes");
      if (attrController.length() > 0) {
        attrs.setAttribute("signature", attrController);
      }
      for (int i = 0; i < attrNames.size(); ++i) {
        XMLElement attr = new XMLElement("attribute");
        String attrName = (String)attrNames.get(i);
        attr.setAttribute("name", attrName);
        attr.setAttribute("value", c.getAttribute(attrName));
        attrs.addSubElement(attr);
      }
      controller.addSubElement(attrs);
    }
  }

  // -------------------------------------------------------------------------
  // Component types
  // -------------------------------------------------------------------------

  /**
   * Stores the type definition converted from the given component in the
   * storage.
   *
   * @param c a component.
   * @param types a map to collect the {@link ComponentType}s of components.
   * @return the name of the type definition converted from c.
   * @throws Exception if something goes wrong.
   */

  public String saveType (final Component c, final Map types) throws Exception {
    ComponentType ctype = new ComponentType(c);
    String name = (String)types.get(ctype);
    if (name != null) {
      return name;
    }
    XMLElement type = new XMLElement("component-type");
    name = getComponentType(c) + "Type";
    type.setAttribute("name", name);
    List itfs = c.getServerInterfaces();
    if (itfs.size() > 0) {
      XMLElement provides = new XMLElement("provides");
      saveInterfaceTypes(itfs, provides);
      type.addSubElement(provides);
    }
    itfs = c.getClientInterfaces();
    if (itfs.size() > 0) {
      XMLElement requires = new XMLElement("requires");
      saveInterfaceTypes(itfs, requires);
      type.addSubElement(requires);
    }
    storage.store(name, type);
    types.put(ctype, name);
    return name;
  }

  /**
   * Converts a list of interfaces into a list of interface type definitions.
   *
   * @param itfs a list of interfaces.
   * @param itfTypes the XML element to which the converted interface types must
   *      be added.
   */

  public void saveInterfaceTypes (final List itfs, final XMLElement itfTypes) {
    for (int i = 0; i < itfs.size(); ++i) {
      Interface itf = (Interface)itfs.get(i);
      if (itf.isCollection() && itf.getMasterCollectionInterface() != null) {
        // do not take into account slave collection interfaces
        continue;
      }
      XMLElement itfType = new XMLElement("interface-type");
      itfType.setAttribute("name", getInterfaceName(itf));
      itfType.setAttribute("signature", itf.getSignature());
      if (itf.isOptional()) {
        itfType.setAttribute("contingency", "optional");
      }
      if (itf.isCollection()) {
        itfType.setAttribute("cardinality", "collection");
      }
      itfTypes.addSubElement(itfType);
    }
  }

  // -------------------------------------------------------------------------
  // Inner classes
  // -------------------------------------------------------------------------

  /**
   * Returns the converted name of the given component. If the component's name
   * is missing, a name is generated for the converted component.
   *
   * @param c a component.
   * @return the converted name of the given component.
   */

  private String getComponentName (final Component c) {
    String s = c.getName();
    if (s.length() == 0) {
      s = "GENERATED-" + Integer.toHexString(c.hashCode());
    }
    return s;
  }

  /**
   * Returns the type of the given component. If the component's type is
   * missing, a name is generated for the converted component.
   *
   * @param c a component.
   * @return the converted type of the given component.
   */

  private String getComponentType (final Component c) {
    String s = c.getType();
    if (s.length() == 0) {
      s = "org.objectweb.fractal.gui.tmp.C" + Integer.toHexString(c.hashCode());
    }
    return s;
  }

  /**
   * Returns the converted name of the given interface. If the interface's name
   * is missing, a name is generated for the converted interface.
   *
   * @param i an interface.
   * @return the converted name of the given interface.
   */

  private String getInterfaceName (Interface i) {
    if (i.isInternal()) {
      i = i.getComplementaryInterface();
    }
    if (i.getMasterInterface() != null) {
      i = i.getMasterInterface();
    }
    String s = i.getName();
    if (i.getMasterCollectionInterface() != null) {
      if (i.getMasterCollectionInterface().getName().length() == 0) {
        s = getInterfaceName(i.getMasterCollectionInterface()) + s;
      }
    } else if (s.length() == 0) {
      s = "GENERATED-" + Integer.toHexString(i.hashCode());
    }
    return s;
  }

  /**
   * A component type.
   */

  class ComponentType {

    /**
     * The types of the interfaces provided by components of this type.
     */

    private Set provides;

    /**
     * The types of the interfaces required by components of this type.
     */

    private Set requires;

    /**
     * Constructs the {@link FractalAdlWriter.ComponentType} of the given
     * component.
     *
     * @param c a component.
     */

    public ComponentType (final Component c) {
      provides = new HashSet();
      requires = new HashSet();
      List l = c.getServerInterfaces();
      for (int i = 0; i < l.size(); ++i) {
        Interface itf = (Interface)l.get(i);
        if (itf.getMasterCollectionInterface() == null) {
          // do not take into account slave collection interfaces
          provides.add(new InterfaceType(itf));
        }
      }
      l = c.getClientInterfaces();
      for (int i = 0; i < l.size(); ++i) {
        Interface itf = (Interface)l.get(i);
        if (itf.getMasterCollectionInterface() == null) {
          // do not take into account slave collection interfaces
          requires.add(new InterfaceType(itf));
        }
      }
    }

    public boolean equals (final Object o) {
      if (o instanceof ComponentType) {
        ComponentType other = (ComponentType)o;
        if (other.provides.equals(provides)) {
          return other.requires.equals(requires);
        }
      }
      return false;
    }

    public int hashCode () {
      int hashCode = 1;
      Iterator i = provides.iterator();
      while (i.hasNext()) {
        hashCode *= i.next().hashCode();
      }
      i = requires.iterator();
      while (i.hasNext()) {
        hashCode *= i.next().hashCode();
      }
      return hashCode;
    }
  }

  /**
   * An interface type.
   */

  class InterfaceType {

    /**
     * The interface from which this interface type is derived.
     */

    private Interface i;

    /**
     * Constructs the {@link FractalAdlWriter.InterfaceType} of the given
     * interface.
     *
     * @param i an interface.
     */

    public InterfaceType (final Interface i) {
      this.i = i;
    }

    public boolean equals (final Object o) {
      if (o instanceof InterfaceType) {
        Interface j = ((InterfaceType)o).i;
        if (getInterfaceName(i).equals(getInterfaceName(j))) {
          if (i.getSignature().equals(j.getSignature())) {
            if (i.isOptional() == j.isOptional()) {
              return i.isCollection() == j.isCollection();
            }
          }
        }
      }
      return false;
    }

    public int hashCode () {
      return getInterfaceName(i).hashCode();
    }
  }
}
