/***
 * FractalGUI: a graphical tool to edit Fractal component configurations.
 * Copyright (C) 2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Authors: Eric Bruneton, Patrice Fauvel
 */

package org.objectweb.fractal.gui.repository.lib;

import org.objectweb.fractal.adl.XMLElement;

import org.objectweb.fractal.gui.repository.api.Storage;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.io.File;
import java.io.LineNumberReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.Writer;

import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.io.OutputStreamWriter;

/**
 * Basic implementation of {@link Storage} interface, based on a single XML
 * file. More precsiely, each storage is a single XML file, and each (name,
 * value) pair is represented by an XML element of these files (the name is
 * given by the 'name' attribute of an XML element).
 */

public class XMLFileStorage implements Storage {

  static String FS = new String (System.getProperty ("file.separator"));

  /**
   * Name of the currently opened storage, or <tt>null</tt> if not storage is
   * opened.
   */

  private String storage;

  /**
   * Content of the currently opened storage. This map associate XML values to
   * names.
   */

  private Map content;

  /**
   * If the currently opened storage has been modified since it has been opened.
   */

  private boolean dirty;

  private String execdir = null;

  public void open (final String storage, final boolean create)
    throws Exception
  {
    if (this.storage != null) {
      throw new Exception("Storage already opened");
    }
    this.storage = storage;
    this.content = new HashMap();
    this.dirty = false;
    File f = new File(storage);
    if (!create) {
      LineNumberReader lnr = new LineNumberReader(new FileReader(f));
      try {
        while (true) {
          XMLElement elem = XMLElement.parse(lnr, false);
          if (elem != null) {
            content.put(elem.getAttribute("name", null), elem);
          } else {
            break;
          }
        }
      } finally {
        lnr.close();
      }
    }
  }

  public Object load (final String name) throws Exception {
    if (storage == null) {
      throw new Exception("Storage not opened");
    }
    XMLElement result = (XMLElement)content.get(name);
    if (result == null) {
      throw new Exception("No such element: " + name);
    }
    return result;
  }

  public void store (final String name, final Object value) throws Exception {
    if (storage == null) {
      throw new Exception("Storage not opened");
    }
    if (!(value instanceof XMLElement)) {
      throw new Exception("Not an XML element: " + value);
    }
    content.put(name, value);
    if (execdir != null) overloadFractalFile (name, value);
    dirty = true;
  }

  public void close () throws Exception {
    if (storage == null) {
      throw new Exception("Storage not opened");
    }
    if (dirty) {
      File f = new File(storage);
      Writer w = new FileWriter(f);
      Iterator i = content.values().iterator();
      while (i.hasNext()) {
        XMLElement elem = (XMLElement)i.next();
        elem.write(w);
        w.write("\n");
      }
      w.close();
    }
    storage = null;
    content = null;
  }
  /**
   * Set the gspecified execution directory which is used if not null
   *
   * @param direxec the name of the execution directory or null otherwise.
   */

  public void setFractalFileDir (String direxec) {
    if (direxec == null) execdir = null;
    else execdir = new String(direxec);
  }

  // -----

  private void overloadFractalFile (String name, Object value) throws Exception
  {
    if (execdir == null) return;
    if (name.equals("FRACTAL_GUI_MAIN")) return;
    String path = new String (execdir+FS+name.replace('.', FS.charAt(0))+
      ".fractal");

    try {
      FileOutputStream fos = new FileOutputStream (path);
      PrintWriter os = new PrintWriter (new OutputStreamWriter (fos));
      os.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\" ?>");
      os.print ("<!DOCTYPE composite-template PUBLIC ");
      os.print ("\"-//objectweb.org//DTD Fractal ADL 1.0//EN\" ");
      os.println ("\"classpath://fractal-adl.dtd\">");
      os.println ();
      os.println ("<!-- overload by FractalGUI -->");
      os.println ();
      os.close();

      // -----
      File f = new File(path);
      Writer w = new FileWriter(f);
      XMLElement elem = (XMLElement)value;
      elem.write(w);
      w.write("\n");
      w.close();
    }
    catch (Exception ex) { throw ex; }
  }
}
